Type: Package
Package: plumber
Title: An API Generator for R
Version: 1.3.3
Authors@R: c(
    person("Barret", "Schloerke", , "barret@posit.co", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0001-9986-114X")),
    person("Jeff", "Allen", , "cran@trestletech.com", role = c("aut", "ccp")),
    person("Bruno", "Tremblay", , "cran@neoxone.com", role = "ctb"),
    person("Frans", "van Dunné", , "frans@ixpantia.com", role = "ctb"),
    person("Sebastiaan", "Vandewoude", , "sebastiaanvandewoude@gmail.com", role = "ctb"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Gives the ability to automatically generate and serve an HTTP
    API from R functions using the annotations in the R documentation
    around your functions.
License: MIT + file LICENSE
URL: https://www.rplumber.io, https://github.com/rstudio/plumber
BugReports: https://github.com/rstudio/plumber/issues
Depends: R (>= 3.0.0)
Imports: crayon, httpuv (>= 1.5.5), jsonlite (>= 0.9.16), lifecycle (>=
        1.0.0), magrittr, mime, promises (>= 1.1.0), R6 (>= 2.0.0),
        rlang (>= 1.0.0), sodium, stringi (>= 0.3.0), swagger (>=
        3.33.0), webutils (>= 1.1)
Suggests: arrow, base64enc, coro, future, geojsonsf, htmlwidgets,
        later, ragg, rapidoc, readr, readxl, redoc, rmarkdown,
        rstudioapi, sf, spelling, svglite, testthat (>= 0.11.0), utils,
        visNetwork, withr, writexl, yaml
ByteCompile: TRUE
Config/Needs/check: Cairo, r-quantities/units
Config/Needs/website: tidyverse/tidytemplate
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Collate: 'async.R' 'content-types.R' 'default-handlers.R' 'hookable.R'
        'shared-secret-filter.R' 'parser-cookie.R' 'parse-body.R'
        'parse-query.R' 'plumber.R' 'deprecated-R6.R' 'deprecated.R'
        'digital-ocean.R' 'find-port.R' 'globals.R' 'includes.R'
        'json.R' 'new-rstudio-project.R' 'openapi-spec.R'
        'openapi-types.R' 'options_plumber.R' 'paths.R' 'plumb-block.R'
        'plumb-globals.R' 'plumb.R' 'plumber-response.R'
        'plumber-static.R' 'plumber-step.R' 'pr.R' 'pr_set.R'
        'serializer.R' 'session-cookie.R' 'ui.R' 'utf8.R'
        'utils-pipe.R' 'utils.R' 'validate_api_spec.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2026-01-27 21:50:00 UTC; barret
Author: Barret Schloerke [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-9986-114X>),
  Jeff Allen [aut, ccp],
  Bruno Tremblay [ctb],
  Frans van Dunné [ctb],
  Sebastiaan Vandewoude [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Barret Schloerke <barret@posit.co>
Repository: CRAN
Date/Publication: 2026-01-28 06:10:08 UTC
