# Generated by roxygen2: do not edit by hand

S3method(format,plumber_available_apis)
S3method(print,plumber_available_apis)
export("%>%")
export(Plumber)
export(PlumberEndpoint)
export(PlumberStatic)
export(addSerializer)
export(as_attachment)
export(available_apis)
export(do_configure_https)
export(do_deploy_api)
export(do_forward)
export(do_provision)
export(do_remove_api)
export(do_remove_forward)
export(endpoint_serializer)
export(forward)
export(getCharacterSet)
export(get_character_set)
export(get_option_or_env)
export(include_file)
export(include_html)
export(include_md)
export(include_rmd)
export(is_plumber)
export(options_plumber)
export(parser_arrow_ipc_stream)
export(parser_csv)
export(parser_excel)
export(parser_feather)
export(parser_form)
export(parser_geojson)
export(parser_json)
export(parser_multi)
export(parser_none)
export(parser_octet)
export(parser_parquet)
export(parser_rds)
export(parser_read_file)
export(parser_text)
export(parser_tsv)
export(parser_yaml)
export(plumb)
export(plumb_api)
export(plumber)
export(pr)
export(pr_cookie)
export(pr_delete)
export(pr_filter)
export(pr_get)
export(pr_handle)
export(pr_head)
export(pr_hook)
export(pr_hooks)
export(pr_mount)
export(pr_post)
export(pr_put)
export(pr_run)
export(pr_set_404)
export(pr_set_api_spec)
export(pr_set_debug)
export(pr_set_docs)
export(pr_set_docs_callback)
export(pr_set_error)
export(pr_set_parsers)
export(pr_set_serializer)
export(pr_static)
export(random_cookie_key)
export(register_docs)
export(register_parser)
export(register_serializer)
export(registered_docs)
export(registered_parsers)
export(registered_serializers)
export(serializer_agg_jpeg)
export(serializer_agg_png)
export(serializer_agg_tiff)
export(serializer_arrow_ipc_stream)
export(serializer_bmp)
export(serializer_cat)
export(serializer_content_type)
export(serializer_csv)
export(serializer_device)
export(serializer_excel)
export(serializer_feather)
export(serializer_format)
export(serializer_geojson)
export(serializer_headers)
export(serializer_html)
export(serializer_htmlwidget)
export(serializer_jpeg)
export(serializer_json)
export(serializer_octet)
export(serializer_parquet)
export(serializer_pdf)
export(serializer_png)
export(serializer_print)
export(serializer_rds)
export(serializer_svg)
export(serializer_svglite)
export(serializer_text)
export(serializer_tiff)
export(serializer_tsv)
export(serializer_unboxed_json)
export(serializer_write_file)
export(serializer_yaml)
export(sessionCookie)
export(session_cookie)
export(validate_api_spec)
import(R6)
import(promises)
import(stringi)
importFrom(crayon,silver)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.set)
importFrom(jsonlite,parse_json)
importFrom(jsonlite,toJSON)
importFrom(lifecycle,deprecated)
importFrom(magrittr,"%>%")
importFrom(rlang,"%||%")
importFrom(rlang,missing_arg)
importFrom(stats,setNames)
importFrom(utils,file.edit)
importFrom(utils,installed.packages)
importFrom(utils,tail)
importFrom(webutils,parse_multipart)
