% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/session-cookie.R
\name{random_cookie_key}
\alias{random_cookie_key}
\title{Random cookie key generator}
\usage{
random_cookie_key()
}
\value{
A 64 digit hexadecimal string to be used as a key for cookie encryption.
}
\description{
Uses a cryptographically secure pseudorandom number generator from \code{\link[sodium:helpers]{sodium::helpers()}} to generate a 64 digit hexadecimal string.  \href{https://github.com/r-lib/sodium}{'sodium'} wraps around \href{https://doc.libsodium.org/}{'libsodium'}.
}
\details{
Please see \code{\link{session_cookie}} for more information on how to save the generated key.
}
\seealso{
\code{\link{session_cookie}}
}
